﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Threading;

namespace HBDotNet
{
    [HBDevice("", 
        Description="",
        WizardFinalText="")]
    public abstract class HBDeviceBase
    {
        protected int _deviceNumber = 0;
        protected IntPtr _deviceHandle = IntPtr.Zero;
        protected HBDeviceAttribute _attr;
        protected bool _enabled = false;
        HBPropertyList _properties = null;

        public int DeviceNumber
        {
            get { return _deviceNumber; }
            internal set { _deviceNumber = value; }
        }

        public IntPtr DeviceHandle
        {
            get { return _deviceHandle; }
            internal set { _deviceHandle = value; }
        }

        public string Name
        {
            get { return _attr.Name; }
        }

        public string Description
        {
            get { return _attr.Description; }
        }

        public string WizardFinalText
        {
            get { return _attr.WizardFinalText; }
        }

        public HBPropertyList Properties
        {
            get { return _properties; }
        }

        #region Wrappers that call HBModule for us

        protected void TraceDeviceMessage(TRC_TYPE trcType, string message)
        {
            HBModule.TraceDeviceMessage(_deviceHandle, trcType, message);
        }

        #region Properties and Values

        protected bool AddArrayPropertyValue(IntPtr hPropertyValue, string newValue)
        {
            return HBModule.AddArrayPropertyValue(hPropertyValue, newValue);
        }

        protected bool ClearArrayPropertyValue(IntPtr hPropertyValue)
        {
            return HBModule.ClearArrayPropertyValue(hPropertyValue);
        }

        protected string GetArrayPropertyValueAsStringAtIndex(IntPtr hPropertyValue, int index)
        {
            return HBModule.GetArrayPropertyValueAsStringAtIndex(hPropertyValue, index);
        }

        #endregion

        #region Change Requests

        protected void ChangeRequest(IntPtr hPropertyValue)
        {
            HBModule.ChangeRequest(hPropertyValue);
        }

        protected void ChangeRequestCompleted(IntPtr hPropertyValue)
        {
            HBModule.ChangeRequestCompleted(hPropertyValue);
        }

        protected void ChangeRequestFailed(IntPtr hPropertyValue)
        {
            HBModule.ChangeRequestFailed(hPropertyValue);
        }

        protected void ChangeRequestAbandoned(IntPtr hPropertyValue)
        {
            HBModule.ChangeRequestAbandoned(hPropertyValue);
        }

        protected void AddNewPropertyValue(IntPtr hProperty, string newValue)
        {
            HBModule.AddNewPropertyValue(hProperty, newValue);
        }
        
        #endregion

        #region Tasks

        protected string[] GetTaskList()
        {
            uint size = 0;
            StringBuilder sb = null;
            bool result = HBModule.GetTaskList( sb, ref size, "\t");
            if (!result) return null;
            if (size == 0) return new string[0];

            sb = new StringBuilder((int)size);
            result = HBModule.GetTaskList(sb, ref size, "\t");
            if (!result) return null;
            return sb.ToString().Split('\t');
        }

        protected bool IsTaskRunning(string taskName)
        {
            return HBModule.IsTaskRunning(taskName);
        }

        protected bool ExecuteTask(string taskName)
        {
            return HBModule.ExecuteTaskByName(taskName);
        }

        protected void StopTask(string taskName)
        {
            HBModule.StopTaskByName(taskName);
        }

        #endregion

        #region Modes

        protected string[] GetModeList()
        {
            uint size = 0;
            StringBuilder sb = null;
            bool result = HBModule.GetModeList(sb, ref size, "\t");
            if (!result) return null;
            if (size == 0) return new string[0];

            sb = new StringBuilder((int)size);
            result = HBModule.GetModeList(sb, ref size, "\t");
            if (!result) return null;
            return sb.ToString().Split('\t');
        }

        protected string GetModeState(string modeName)
        {
            return HBModule.GetModeStateByName(modeName);
        }

        protected bool SetModeState(string modeName, bool active)
        {
            return HBModule.SetModeStateByName(modeName, active);
        }

        #endregion

        /*
         * Not implemented
         * 
        public delegate IntPtr dlgtCreatePropertyValue(IntPtr hProperty, string szNewValue);
        
        public delegate void dlgtResetSunRiseAndSunSetTimes();
        public delegate bool dlgtGetDevicePropertyValueFromPropertyHandle(IntPtr hProperty, out IntPtr hPropertyValue, out string szPropertyValue, out string szConfigValue);
        public delegate bool dlgtGetDevicePropertyValueFromHandle(IntPtr hProperyValue, out string szPropertyValue, out string szConfigValue);
        public delegate bool dlgtRegisterDeviceProperty(IntPtr hDeviceModule, int nDeviceNumber, string szPropertyName, string szPropertyDescription, bool bDefault, string szInitialValue, eIOType ioType, bool bPersistent, bool bAllowSameValueChange, bool bTraceToDeviceLog, bool bDynamic);
        public delegate bool dlgtGetPropertyInfoFromHandle(IntPtr hProperty, StringBuilder szPropertyName, StringBuilder szPropertyDescription);
        public delegate IntPtr dlgtGetDevicePropertyHandleFromName(IntPtr hDevice, string szPropertyName, StringBuilder szPropertyDescription);
        
        public delegate bool dlgtGetIRCodeArrayAssociatedWithPropertyValue(IntPtr hProperty, IntPtr hPropertyValue, ref ushort[] pRetArray, ref uint pdwBufferSize);
        public delegate IntPtr dlgtGetPropertyHandleFromPropertyValueHandle(IntPtr hPropertyValue);
        public delegate HARDWARE_RC dlgtSendInterfacePackToInterface(IntPtr hDevice, IntPtr pPack);
        public delegate void dlgtSubscribeToNotificationList(IntPtr hDevice, string szListName, string szFilter);
        public delegate void dlgtUnSubscribeFromNotificationList(IntPtr hDevice, string szListName, string szFilter);
        public delegate bool dlgtGetDeviceInfoFromDescription(string szDeviceDescription, out IntPtr hDevice, out bool bEnabled, out string szDeviceName);
        public delegate bool dlgtGetPropertyInfoByDescription(IntPtr hDevice, string szPropertyDescription, out IntPtr hProperty, StringBuilder szPropertyName);

        //public delegate bool dlgtGetWinMsgArrayAssociatedWithPropertyValue(....
        public delegate bool dlgtCreateDeviceProperty(string szPropertyName, string szPropertyDescription, ePropertyType ptType);
        public delegate bool dlgtCreateDevicePropertyValue(string szPropertyName, string szPropertyValue);
        public delegate bool dlgtCreateDevicePropertyMapping(IntPtr hDevice, string szPropertyName, string szPropertyDescription, string szInitialValue, eIOType ioType, bool bDefault, bool bPersistent, bool bAllowSameValueChange, bool bTraceToDevLog, bool bDynamic);
        public delegate int dlgtGetDeviceID(IntPtr hDevice);
        public delegate bool dlgtAddNewPropertyValue(IntPtr hProperty, string szNewValue);
        //public delegate bool dlgtGetGenericCommansAssociatedWithValue(...
        public delegate bool dlgtQueryHardwareInstance(IntPtr hDevice, string szQuery, IntPtr pResponse);
        public delegate bool dlgtControlRemote(eClientControlAction ccaAction, string szWhichRemotes, string pArg1, string pArg2);
        public delegate bool dlgtGetDeviceList(StringBuilder pszDeviceList, ref uint pdwBufferSize, string szDelimiter);
        public delegate bool dlgtGetPropertyListForDevice(string szDeviceDescription, StringBuilder pszPropertyList, ref uint pdwBufferSize, string szDelimiter);
        public delegate bool dlgtGetPropertyValueListForProperty(string szPropertyDescription, StringBuilder pszValueList, ref uint pdwBufferSize, string szDelimiter);
        
        public delegate bool dlgtRegisterDevicePropertyUsingWizard(IntPtr hDeviceModule, int nDeviceNumber, string szPropertyName, string szPropertyDescription, bool bDefault, string szInitialValue, eIOType ioType, bool bPersistent, bool bAllowSameValueChange, bool bTraceToDeviceLog, bool bDynamic, string szWizardText);
        public delegate bool dlgtRegisterDeviceButton(IntPtr hDeviceModule, int nDeviceNumber, int nButtonCallbackID, string szButtonHeading, string szButtonText);
        public delegate bool dlgtGetAndRemoveNextChangeRequestOnQueue2(IntPtr hDevice, out IntPtr phProperty, out IntPtr phPropertyValue, StringBuilder szPropertyName, StringBuilder szPropertyDescription, StringBuilder szPropertyValue, ref int nPropertyValueLen, StringBuilder szConfigValue, ref int nConfigValueLen);
        public delegate bool dlgtSubscribeToPropertyChange(IntPtr hDevice, string szDeviceDescription, string szPropertyDescription);
        public delegate bool dlgtUnsubscribeFromPropertyChange(IntPtr hDevice, string szDeviceDescription, string szPropertyDescription);
        
        public delegate bool dlgtGetIRNumericArrayForRemoteAssociatedToValue(IntPtr hProperty, IntPtr hPropertyValue, ref ushort[] pRetArray);
        */

        #endregion
        
        public HBDeviceBase()
        {
            HBDeviceAttribute[] attrs = (HBDeviceAttribute[])this.GetType().GetCustomAttributes(typeof(HBDeviceAttribute), true);
            if (attrs.Length > 0)
            {
                _attr = attrs[0];
            }
            _properties = new HBPropertyList(this);
        }

        /// <summary>
        /// OnRegisterProperties: Called by HouseBot to ask the device to register its properties.
        /// The base class registers the properties that are defined by applying the HBPropertyAttribute attribute on the class.
        /// If you override this method, you should create your own properties (HBProperty) and add them to to the Properties list.
        /// Calling the base class is then optional.
        /// </summary>
        /// <returns>Success/Failure</returns>
        public virtual bool OnRegisterProperties()
        {
            // By default, derive the properties from the HBPropertyAttribute that's applied to our class.
            // Device implementors may decide to override this method to change this behavior.

            HBPropertyAttribute[] attrs = (HBPropertyAttribute[]) this.GetType().GetCustomAttributes(typeof(HBPropertyAttribute), false);
            foreach (HBPropertyAttribute attr in attrs)
            {
                HBProperty property = HBProperty.FromHBPropertyAttribute(attr);
                _properties.Add(property);
            }
            return true;
        }

        internal DeviceInfo GetDeviceInfo()
        {
            DeviceInfo info = new DeviceInfo();

            //
            // derive the base info from the HBDeviceAttribute
            //
            info.m_szDeviceName = this.Name;
            info.m_eDeviceType = eDeviceType.dtSoftware;
            info.m_szModuleWizardDescription = this.Description;
            info.m_szWizardFinalText = this.WizardFinalText;
            info.m_dfFlags = eDeviceFlags.dfNoFlags;

            // 
            // See if the derived class overrides "DisplayHelp" and set the "HelpSupported" flag appropriately
            // 
            MethodInfo mi = this.GetType().GetMethod("DisplayHelp");
            if ((mi!=null) && (mi.DeclaringType != typeof(HBDeviceBase)))
            {
                info.m_dfFlags |= eDeviceFlags.dfHelpSupported;
            }
            else
            {
                info.m_dfFlags |= eDeviceFlags.dfHelpNOTSupported;
            }
            return info;
        }

        /// <summary>
        /// Called when your device is initializing. Allocate resources in "OnInit", free them in "OnShutDown".
        /// The base class simply copies the "enabled" parameter and returns true.
        /// </summary>
        /// <param name="enabled"></param>
        /// <returns>Success or Failure. </returns>
        public virtual bool OnInit(bool enabled)
        {
            _enabled = enabled;
            return true;
        }

        /// <summary>
        /// Called when the user enables your device. 
        /// The base class simply returns true.
        /// </summary>
        /// <returns>True if your device successfully enabled itself. False otherwise.</returns>
        public virtual bool OnEnable()
        {
            return true;
        }

        /// <summary>
        /// Called when the user enables your device. 
        /// The base class simply returns true.
        /// </summary>
        /// <returns>True if your device successfully disable itself. False otherwise.</returns>
        public virtual bool OnDisable()
        {
            return true;
        }

        internal bool SetEnabled(bool enabled)
        {
            if ((!_enabled) && (enabled))
            {
                if (OnEnable())
                {
                    _enabled = enabled;
                    return true;
                }
                else
                {
                    return false;
                }
            }
            if ((_enabled) && (!enabled))
            {
                if (OnDisable())
                {
                    _enabled = enabled;
                    return true;
                }
                else
                {
                    return false;
                }
            }
            _enabled = enabled;
            return true;
        }

        private WaitHandle NativeHandleToWaitHandle(IntPtr h)
        {
            System.Threading.EventWaitHandle result = new EventWaitHandle(false, EventResetMode.ManualReset);
            result.SafeWaitHandle = new Microsoft.Win32.SafeHandles.SafeWaitHandle(h, false);
            return result;
        }

        internal bool Run(IntPtr hTerminate, IntPtr hPropertyChangeRequest)
        {
            WaitHandle[] handles = new WaitHandle[2];
            handles[0] = NativeHandleToWaitHandle(hTerminate);
            handles[1] = NativeHandleToWaitHandle(hPropertyChangeRequest);
            while (true)
            {
                int waitResult = WaitHandle.WaitAny(handles);
                switch( waitResult )
                {
                    case 0: // terminate
                        OnShutdown();
                        return true;

                    case 1: // property change request
                        IntPtr hProperty = IntPtr.Zero;
                        IntPtr hPropertyValue = IntPtr.Zero;
                        StringBuilder propertyName = new StringBuilder(1024);
                        StringBuilder propertyDescription = new StringBuilder(1024);
                        StringBuilder propertyValue = new StringBuilder(1024);
                        int propertyValueLen = 1024;
                        StringBuilder configValue = new StringBuilder(1024);
                        int configValueLen = 1024;

                        bool result = HBModule.GetAndRemoveNextChangeRequestOnQueue2(
                            _deviceHandle,
                            out hProperty,
                            out hPropertyValue,
                            propertyName,
                            propertyDescription,
                            propertyValue,
                            ref propertyValueLen,
                            configValue,
                            ref configValueLen);

                        if (result)
                        {
                            HBProperty prop = Properties.GetPropertyByHandle(hProperty);
                            result = OnPropertyChangeRequest(prop, propertyValue.ToString());
                            if (result) prop.internalSetCurrentValue(propertyValue.ToString());
                        }
                        if (result)
                        {
                            HBModule.ChangeRequestCompleted(hPropertyValue);
                        }
                        else
                        {
                            HBModule.ChangeRequestFailed(hPropertyValue);
                        }
                        break;
                }
            }
        }

        /// <summary>
        /// Called when HB wants to change a property.
        /// </summary>
        /// <param name="property"></param>
        /// <param name="newValue"></param>
        /// <returns>Return true to accept this change; false to reject it. The base class simply returns true.</returns>
        protected virtual bool OnPropertyChangeRequest(HBProperty property, string newValue)
        {
            return true;
        }

        /// <summary>
        /// Called by HB when the user clicks the "Help" button.
        /// You must set the "HelpSupported" field in the HBDeviceAttribute for the help button to be enabled.
        /// The base class does nothing.
        /// </summary>
        public virtual void OnDisplayHelp()
        {
        }

        /// <summary>
        /// Called by HB when the user clicks the "About" button.
        /// The base class displays a simple message box with the name and description of your device.
        /// </summary>
        public virtual void OnAboutBox()
        {
            System.Windows.Forms.MessageBox.Show(
                Description,
                Name,
                System.Windows.Forms.MessageBoxButtons.OK,
                System.Windows.Forms.MessageBoxIcon.Information,
                System.Windows.Forms.MessageBoxDefaultButton.Button1);
        }

        /// <summary>
        /// Called by HB when the user clicks a button.
        /// The base class does nothing.
        /// </summary>
        /// <param name="buttonID"></param>
        public virtual void OnButtonPressNotification(int buttonID)
        {
        }

        /// <summary>
        /// DO NOT USE THIS YET!! dataPack isn't supported.
        /// </summary>
        /// <param name="listName"></param>
        /// <param name="filter"></param>
        /// <param name="dataPack"></param>
        /// <returns></returns>
        public virtual bool OnSubscriptionNotification(string listName, string filter, IntPtr dataPack)
        {
            return false;
        }

        /// <summary>
        /// If you've subscribed to PropertyChangeNotifications, HB calls this to notify you of property changes.
        /// The base class does nothing.
        /// </summary>
        /// <param name="deviceDescription"></param>
        /// <param name="propertyDescription"></param>
        /// <param name="newValue"></param>
        public virtual void OnPropertyChangeNotification(string deviceDescription, string propertyDescription, string newValue)
        {
        }

        /// <summary>
        /// Called when your device is shutting down. Allocate resources in "OnInit", free them in "OnShutDown".
        /// The base class does nothing.
        /// </summary>
        public virtual void OnShutdown()
        {
        }

    }
}
