﻿using System;
using System.Collections.Generic;
using System.Text;

namespace HBDotNet
{
    public class HBProperty
    {
        protected HBDeviceBase _owner = null;
        protected IntPtr _handle = IntPtr.Zero;
        protected string _name = string.Empty;
        protected string _description = string.Empty;
        protected bool _isDefault = false;
        protected string _initialValue = string.Empty;
        protected eIOType _ioType = eIOType.ioUnknown;
        protected bool _isPersistent = false;
        protected bool _allowSameValueChange = false;
        protected bool _traceToDeviceLog = false;
        protected bool _isDynamic = false;
        protected ePropertyType _propertyType = ePropertyType.ptAlpha;
        protected string _wizardText = string.Empty;
        protected string _currentValue = string.Empty;
        protected bool _currentValueInitialized = false;

        public HBDeviceBase Owner
        {
            get{ return _owner; }
            internal set { _owner = value; }
        }

        public IntPtr Handle
        {
            get 
            {
                if (_handle == IntPtr.Zero)
                {
                    StringBuilder propdescr = new StringBuilder(1024);
                    _handle = HBModule.GetDevicePropertyHandleFromName(_owner.DeviceHandle, _name, propdescr);
                }
                return _handle; 
            }
        }

        public string Name
        {
            get { return _name; }
        }

        public string Description
        {
            get { return _description; }
        }

        public bool IsDefault
        {
            get { return _isDefault; }
        }

        public string InitialValue
        {
            get { return _initialValue; }
        }

        public eIOType IOType
        {
            get { return _ioType; }
        }

        public bool IsPersistent
        {
            get { return _isPersistent; }
        }

        public bool AllowSameValueChange
        {
            get { return _allowSameValueChange; }
        }

        public bool TraceToDeviceLog
        {
            get { return _traceToDeviceLog; }
        }

        public bool IsDynamic
        {
            get { return _isDynamic; }
        }

        public ePropertyType PropertyType
        {
            get { return _propertyType; }
        }

        public string WizardText
        {
            get { return _wizardText; }
        }

        public string CurrentValue
        {
            get 
            {
                if (!_currentValueInitialized)
                {
                    IntPtr hPropertyValue;
                    string configValue;
                    if (HBModule.GetDevicePropertyValueFromPropertyHandle(Handle, out hPropertyValue, out _currentValue, out configValue))
                    {
                        _currentValueInitialized = true;
                    }
                }
                return _currentValue; 
            }
            set 
            { 
                // create a new PropertyValueHandle for this new value
                IntPtr hPropertyValue = HBModule.CreatePropertyValue(_handle, value);
                HBModule.ChangeRequestCompleted(hPropertyValue);
                _currentValue = value; 
            }
        }

        internal void internalSetCurrentValue(string newValue)
        {
            _currentValue = newValue;
        }

        public HBProperty(string name, ePropertyType propertyType, string initialValue)
            : this(
                name, 
                name,
                false, // isDefault
                initialValue,
                eIOType.ioInputAndOutput,
                false, // isPersistent
                false, // allowSameValueChanges
                false, // traceToDeviceLog
                false, // isDynamic
                propertyType,
                string.Empty // wizardText
            )
        {
        }

        public HBProperty(string name, string description, bool isDefault, string initialValue, eIOType ioType, bool isPersistent, bool allowSameValueChange, bool traceToDeviceLog, bool isDynamic, ePropertyType ptType, string wizardText)
        {
            _name = name;
            _description = description;
            _isDefault = isDefault;
            _initialValue = initialValue;
            _ioType = ioType;
            _isPersistent = isPersistent;
            _allowSameValueChange = allowSameValueChange;
            _traceToDeviceLog = traceToDeviceLog;
            _isDynamic = isDynamic;
            _wizardText = wizardText;
            _currentValue = _initialValue; 
        }

        public bool Register()
        {
            return HBModule.RegisterAndCreateDeviceProperty(
                 HBModule.ModuleHandle,
                 _owner.DeviceNumber,
                 _name,
                 _description,
                 _isDefault,
                 _initialValue,
                 _ioType,
                 _isPersistent,
                 _allowSameValueChange,
                 _traceToDeviceLog,
                 _isDynamic,
                 _propertyType,
                 _wizardText);
        }

        internal static HBProperty FromHBPropertyAttribute(HBPropertyAttribute attr)
        {
            HBProperty prop = new HBProperty(
                attr.Name,
                attr.Description,
                attr.IsDefault,
                attr.InitialValue,
                attr.IOType,
                attr.IsPersistent,
                attr.AllowSameValueChange,
                attr.TraceToDeviceLog,
                attr.IsDynamic,
                attr.PropertyType,
                attr.WizardText);
            return prop;
        }

    }
}
