﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace HBDotNet
{
    // Trace message types
    public enum TRC_TYPE
    {
        ttError,          // Trace as error message
        ttWarning,        // Trace as warning mesage
        ttDebug           // Trace as debug message
    };

    // Device types
    public enum eDeviceType
    {
        dtUnknown,           // Reserved, don't use
        dtHardware,          // Hardware device
        dtSoftware,          // Software device
        dtLastDeviceType     // Reserved, don't use
    };

    // Input / Output types for properties.
    // !! These are currenly NOT supported !!
    public enum eIOType
    {
        ioUnknown,           // Reserved, don't use
        ioInputOnly,         // Input ONLY property
        ioOutputOnly,        // Output ONLY property
        ioInputAndOutput,    // Input and output property
        ioLastIO             // Reserved, don't use
    };

    // Valid property types.
    public enum ePropertyType
    {
        ptNumeric,
        ptAlpha,
        ptTime,
        ptTimeSpan,
        ptBoolean,
        ptHouseCode,
        ptUnitCode,
        ptAlphaList,
        ptPath,
        ptFileName,
        ptPathAndFileName,
        ptLastPropertyType
    };

    // Windows Message Type identifier for WinMsg structure
    public enum eWinMsgType
    {
        wmtUnknown,
        wmtPause,
        wmtKeyStroke,
        wmtSpecialKey,
        wmtLastMessageType
    };

    // Windows Message Key state identifier for WinMsg structure
    public enum eWinMsgKeyStates
    {
        wmksAlt = 0x1,
        wmksCtl = 0x2,
        wmksSft = 0x4,
        wmksPost = 0x8
    };

    public enum eWinMsgMouseEvents
    {
        meUnknown,
        meUp,
        meDown,
        meLeft,
        meRight,
        meLeftClick,
        meRightClick,
        meLastMouseEvent
    };

    /*
    // Windows Message definition
    struct WinMsg
    {
       WinMsg() { m_wmtMessageType = m_wmksKeyStates = m_nValue = 0; }
       WORD        m_wmtMessageType;             // Type of Windows Message data contained in structure.  See eWinMsgType.
       WORD        m_wmksKeyStates;              // Key states.  See eWinMsgKeyStates.
       int         m_nValue;                     // Type specific value.
    };
        
    // Generic Command definition
    struct GenericCommand
    {
       GenericCommand() { m_szCommand = NULL; }
       const char* m_szCommand;                  // Command string.
    };
    */

    // Device flag bit mask values
    [FlagsAttribute]
    public enum eDeviceFlags
    {
        dfNoFlags = 0x00,
        dfWindowsMessageSupport = 0x01,              // Indicates that the plugin supports Windows Messages.  This causes the server to
        // display the Windows Messages mapping property page when configuring property values.
        // These two flags control help support.  If neither of them are defined, HB determines
        // help support by the presense of the HBDisplayHelp export.
        dfHelpSupported = 0x02,              // Indicates that the plugin supports on-line help.
        dfHelpNOTSupported = 0x04,              // Indicates that the plugin does NOT support on-line help.

        dfUseMeedioAboutBox = 0x08,              // Backward Compatibility ,,,, Don't use for new plugins.
        dfUseDefaultAboutBox = 0x08,              // Used for CeBotics Plugins only.  Displays default CeBotics about box.
    };


    // DeviceInfo structure used to return Device specific
    // information to the server.
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct DeviceInfo
    {
        [MarshalAs(UnmanagedType.I4)]
        public int m_nDeviceInfoSize;            // Intialized at runtime with the sizeof() the structure
        [MarshalAs(UnmanagedType.LPStr)]
        public string m_szDeviceName;               // Name of the Device
        [MarshalAs(UnmanagedType.I4)]
        public eDeviceType m_eDeviceType;                // Type (hardware or software) of the Device
        [MarshalAs(UnmanagedType.LPStr)]
        public string m_szInterfaceSignatures;      // For hardware devices, list of signagures requried to support the device.  Concatenate interface signagures with "|"
        [MarshalAs(UnmanagedType.I4)]
        public eDeviceFlags m_dfFlags;                    // Additional flag values.
        [MarshalAs(UnmanagedType.LPStr)]
        public string m_szModuleWizardDescription;  // Description of the plugin.  Used in Wizard.
        [MarshalAs(UnmanagedType.LPStr)]
        public string m_szWizardFinalText;          // Text displayed at the end of the Wizard process.
    };

    // Hardware Interface return codes
    public enum HARDWARE_RC
    {
        hrcFailureRetry,  // Call has failed, but you should probably retry.
        hrcFailureAbort,  // Call has failed, please don't retry.
        hrcSuccess        // Call has succeeded.
    };

    // Enum for pHB_ControlRemote callback
    public enum eClientControlAction
    {
        ccaOpenPanel,
        ccaClosePanel,
        ccaExecuteApp,
        ccaMinimize,
        ccaRestore,
        ccaBlankScreen,
        ccaControlFocus,

        ccaUnknown
    };

    // Enum for pHB_ControlRemote callback (pArg1 if dealing with control focus)
    public enum eFocusControl
    {
        fcFocusUp,
        fcFocusDown,
        fcFocusLeft,
        fcFocusRight,
        fcFocusSetValue,
        fcFocusEnter
    };

}